pub struct AddressChecksumBuffer(/* private fields */);Expand description
Stack-allocated buffer for efficiently computing address checksums.
See Address::to_checksum_buffer for more information.
Implementations
Sourceimpl AddressChecksumBuffer
impl AddressChecksumBuffer
Sourcepub const unsafe fn new() -> AddressChecksumBuffer
pub const unsafe fn new() -> AddressChecksumBuffer
Creates a new buffer.
Safety
The buffer must be initialized with format before use.
Prefer Address::to_checksum_buffer instead.
Sourcepub fn format(&mut self, address: &Address, chain_id: Option<u64>) -> &mut str
pub fn format(&mut self, address: &Address, chain_id: Option<u64>) -> &mut str
Calculates the checksum of an address into the buffer.
See Address::to_checksum_buffer for more information.
Sourcepub const fn as_mut_str(&mut self) -> &mut str
pub const fn as_mut_str(&mut self) -> &mut str
Returns the checksum of a formatted address.
Sourcepub const fn into_inner(self) -> [u8; 42]
pub const fn into_inner(self) -> [u8; 42]
Returns the backing buffer.